import router from '@/router'

type Method = 'GET' | 'POST' | 'PUT' | 'DELETE'
type ResponseType = 'arraybuffer' | 'blob' | 'document' | 'json' | 'text' | 'stream'

interface AxiosRequest {
  baseURL?: string
  url: string
  data?: any
  params?: any
  method?: Method
  headers?: any
  timeout?: number
  responseType?: ResponseType
}

import axios from 'axios'

// 定义接口
interface PendingType {
  url?: string
  method?: Method
  params: any
  data: any
  cancel: Function
}

// 取消重复请求
const pending: Array<PendingType> = []
const CancelToken = axios.CancelToken

// axios 实例
const instance = axios.create({
  timeout: 60000,
  responseType: 'json'
})

// 添加请求拦截器
instance.interceptors.request.use(
  (request: any) => {
    // TODO: handle loading
    request.headers['Authorization'] = 'Bearer ' + sessionStorage['token']
    return request
  },
  (error: any) => {
    return Promise.reject(error)
  }
)

const errorHandle = (error: any) => {
  // ElMessage.error(error?.msg || '服务器错误')
  switch (error?.status) {
    case 401:
      sessionStorage['token'] = ''
      router.push('/login')
      // token失效
      break
    case 403:
      // 没有权限
      break
    case 500:
      // 服务端错误
      break
    case 503:
      // 服务端错误
      break
    default:
      break
  }
}
// 添加响应拦截器
instance.interceptors.response.use(
  (response: any) => {
    return response
  },
  (error: any) => {
    const response = error.response

    // 根据返回的http状态码做不同的处理
    errorHandle(response)

    return Promise.reject(response || { msg: error.msg })
  }
)

class BaseHttp {
  // 外部传入的baseUrl
  private baseURL: string = ''
  // 自定义header头
  private headers: object = {
    ContentType: 'application/json;charset=UTF-8'
  }

  private apiAxios({
    baseURL = this.baseURL,
    headers = this.headers,
    method,
    url,
    data,
    params,
    responseType
  }: AxiosRequest): Promise<any> {
    return new Promise((resolve, reject) => {
      instance({
        baseURL,
        headers,
        method,
        url,
        params,
        data,
        responseType
      })
        .then((res: any) => {
          // 200:服务端业务处理正常结束
          resolve(res.data)
          // if (res.data.code == 200 || res.data.type === 'application/octet-stream') {
          //   // TODO ...
          //   resolve(res.data)
          // } else {
          //   console.log(res?.data?.code, 'res?.data?.code res?.data?.code ')
          //   errorHandle({
          //     status: res?.data?.code || 500,
          //     msg: res.data?.msg || url + '请求失败',
          //     data: null
          //   })
          //   reject({ status: false, msg: res.data?.msg || url + '请求失败', data: null })
          // }
        })
        .catch((err: any) => {
          console.log('🚀 ~ BaseHttp ~ returnnewPromise ~ err:', err)
          const msg = err?.data?.msg || err?.msg || url + '请求失败'
          // eslint-disable-next-line
          reject({ status: false, msg, data: null })
        })
    })
  }

  /**
   * GET类型的网络请求
   */
  get({ baseURL, headers, url, data, params, responseType }: AxiosRequest) {
    return this.apiAxios({
      baseURL,
      headers,
      method: 'GET',
      url,
      data,
      params,
      responseType
    })
  }

  /**
   * POST类型的网络请求
   */
  post({ baseURL, headers, url, data, params, responseType }: AxiosRequest) {
    return this.apiAxios({
      baseURL,
      headers,
      method: 'POST',
      url,
      data,
      params,
      responseType
    })
  }

  /**
   * PUT类型的网络请求
   */
  put({ baseURL, headers, url, data, params, responseType }: AxiosRequest) {
    return this.apiAxios({
      baseURL,
      headers,
      method: 'PUT',
      url,
      data,
      params,
      responseType
    })
  }

  /**
   * DELETE类型的网络请求
   */
  delete({ baseURL, headers, url, data, params, responseType }: AxiosRequest) {
    return this.apiAxios({
      baseURL,
      headers,
      method: 'DELETE',
      url,
      data,
      params,
      responseType
    })
  }
}

export default new BaseHttp()
